@@ -43,10 +43,18 @@ classdef Kriging < handle
             [varargout{1:nargout}] = mLibKriging("Kriging::summary", obj.ref, varargin{:});
         end
 
+        function varargout = save(obj, varargin)
+            [varargout{1:nargout}] = mLibKriging("Kriging::save", obj.ref, varargin{:});
+        end
+
         function varargout = leaveOneOutFun(obj, varargin)
             [varargout{1:nargout}] = mLibKriging("Kriging::leaveOneOutFun", obj.ref, varargin{:});
         end
 
+        function varargout = leaveOneOutVec(obj, varargin)
+            [varargout{1:nargout}] = mLibKriging("Kriging::leaveOneOutVec", obj.ref, varargin{:});
+        end
+
         function varargout = leaveOneOut(obj, varargin)
             [varargout{1:nargout}] = mLibKriging("Kriging::leaveOneOut", obj.ref, varargin{:});
         end
@@ -152,4 +160,11 @@ classdef Kriging < handle
         end
 
     end
+
+    methods (Static = true)
+        function obj = load(varargin)
+            obj = Kriging([1], [1], "gauss") % TODO should find a more straightforward default ctor
+            obj.ref = mLibKriging("Kriging::load", varargin{:});
+        end
+    end
 end
