@@ -12,6 +12,8 @@ void predict(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
 void simulate(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
 void update(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
 void summary(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
+void save(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
+void load(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
 void logLikelihoodFun(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
 void logMargPostFun(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
 void logLikelihood(int nlhs, mxArray** plhs, int nrhs, const mxArray** prhs);
