@@ -110,10 +110,18 @@ std::string PyNoiseKriging::summary() const {
   return m_internal->summary();
 }
 
+void PyNoiseKriging::save(const std::string filename) const {
+  return m_internal->save(filename);
+}
+
+PyNoiseKriging PyNoiseKriging::load(const std::string filename) {
+  return PyNoiseKriging(std::make_unique<NoiseKriging>(NoiseKriging::load(filename)));
+}
+
 std::tuple<double, py::array_t<double>> PyNoiseKriging::logLikelihoodFun(const py::array_t<double>& theta_sigma2,
                                                                          const bool want_grad) {
   arma::vec vec_theta_sigma2 = carma::arr_to_col<double>(theta_sigma2);
-  auto [llo, grad] = m_internal->logLikelihoodFun(vec_theta_sigma2, want_grad);
+  auto [llo, grad] = m_internal->logLikelihoodFun(vec_theta_sigma2, want_grad, false);
   return {llo, carma::col_to_arr(grad)};
 }
 
