@@ -93,3 +93,29 @@ def test_data2(i):
     ll, llgrad, llhess = r.logLikelihoodFun(x, True, False)
     assert relative_error(ll, ll_ref) < tolerance
     assert relative_error(llgrad, llgrad_ref) < tolerance
+
+
+@pytest.mark.parametrize("i", np.arange(1, 11))
+def test_data2_with_save_reload(i):
+    refpath = find_dir()
+    prefix = f"data2-grad-{i}"
+    filex = refpath / f"{prefix}-X.csv"
+    filey = refpath / f"{prefix}-y.csv"
+    X = np.genfromtxt(filex, delimiter=',')
+    y = np.genfromtxt(filey, delimiter=',').reshape(-1, 1)  # FIXME should be a col vec
+    file_ll = refpath / f"{prefix}-result-logLikelihood.csv"
+    file_llgrad = refpath / f"{prefix}-result-logLikelihoodGrad.csv"
+    ll_ref = np.genfromtxt(file_ll, delimiter=',')
+    llgrad_ref = np.genfromtxt(file_llgrad, delimiter=',').reshape(-1, 1)  # FIXME should be a col vec
+
+    kernel = "gauss"
+    r = lk.Kriging(y, X, kernel)  # use defaults: lk.RegressionModel.Constant, False, "BFGS", "LL", lk.Parameters())
+
+    r.save("test_data2_with_save_reload.h5");
+    r = lk.Kriging.load("test_data2_with_save_reload.h5");
+
+    x = 0.3 * np.ones(np.shape(X)[1])
+
+    ll, llgrad, llhess = r.logLikelihoodFun(x, True, False)
+    assert relative_error(ll, ll_ref) < tolerance
+    assert relative_error(llgrad, llgrad_ref) < tolerance
