@@ -24,11 +24,12 @@ function repair_wheel {
 
 
 # Install a system package required by our library
-yum install -y openblas-devel
+yum install -y openblas-devel hdf5-devel
 
 # Compile wheels
 for PYBIN in /opt/python/*-cp*/bin; do
     if [ "${PYBIN}" == "/opt/python/cp35-cp35m/bin" ]; then continue; fi
+    if [ "${PYBIN}" == "/opt/python/cp312-cp312/bin" ]; then continue; fi
     "${PYBIN}/pip" install -r "${ROOT_DIR}"/bindings/Python/requirements.txt # not for Windows
     "${PYBIN}/pip" install -r "${ROOT_DIR}"/bindings/Python/dev-requirements.txt # not for Windows
     "${PYBIN}/python3" "${ROOT_DIR}"/bindings/Python/setup.py bdist_wheel
@@ -42,6 +43,7 @@ done
 # Install packages and test
 for PYBIN in /opt/python/*-cp*/bin; do
     if [ "${PYBIN}" == "/opt/python/cp35-cp35m/bin" ]; then continue; fi
+    if [ "${PYBIN}" == "/opt/python/cp312-cp312/bin" ]; then continue; fi
     "${PYBIN}/pip" install pylibkriging --no-index -f "${ROOT_DIR}"/dist
     (cd "${ROOT_DIR}"; "${PYBIN}/pytest" "${ROOT_DIR}"/bindings/Python/tests)
 done
