@@ -1,10 +1,11 @@
 ## ****************************************************************************
 ## This file contains stuff related to the S3 class "NuggetKriging".
-## As an S3 class, it has no formal definition. 
+## As an S3 class, it has no formal definition.
 ## ****************************************************************************
 
 
-## ****************************************************************************
+
+
 #' Create an object with S3 class \code{"NuggetKriging"} using
 #' the \pkg{libKriging} library.
 #'
@@ -12,11 +13,9 @@
 #' are estimated thanks to the optimization of a criterion given by
 #' \code{objective}, using the method given in \code{optim}.
 #'
-#' @title Create a \code{NuggetKriging} Object using \pkg{libKriging}
-#' 
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
-#' @param y Numeric vector of response values. 
+#'
+#' @param y Numeric vector of response values.
 #'
 #' @param X Numeric matrix of input design.
 #'
@@ -30,7 +29,7 @@
 #'     values in the interval \eqn{[0, 1]}.
 #'
 #' @param optim Character giving the Optimization method used to fit
-#'     hyper-parameters. Possible values are: \code{"BFGS"} and \code{"none"}, 
+#'     hyper-parameters. Possible values are: \code{"BFGS"} and \code{"none"},
 #'     the later simply keeping
 #'     the values given in \code{parameters}. The method
 #'     \code{"BFGS"} uses the gradient of the objective.
@@ -38,17 +37,17 @@
 #' @param objective Character giving the objective function to
 #'     optimize. Possible values are: \code{"LL"} for the
 #'     Log-Likelihood and \code{"LMP"} for the Log-Marginal Posterior.
-#' 
+#'
 #' @param parameters Initial values for the hyper-parameters. When provided this
-#'     must be named list with some elements \code{"sigma2"}, \code{"theta"}, \code{"nugget"} 
-#'     containing the initial value(s) for the variance, range and nugget 
-#'     parameters. If \code{theta} is a matrix with more than one row, 
+#'     must be named list with some elements \code{"sigma2"}, \code{"theta"}, \code{"nugget"}
+#'     containing the initial value(s) for the variance, range and nugget
+#'     parameters. If \code{theta} is a matrix with more than one row,
 #'     each row is used as a starting point for optimization.
-#' 
+#'
 #' @return An object with S3 class \code{"NuggetKriging"}. Should be used
 #'     with its \code{predict}, \code{simulate}, \code{update}
 #'     methods.
-#' 
+#'
 #' @export
 #' @useDynLib rlibkriging, .registration = TRUE
 #' @importFrom Rcpp sourceCpp
@@ -62,20 +61,20 @@
 #' ## fit and print
 #' k <- NuggetKriging(y, X, kernel = "matern3_2")
 #' print(k)
-#' 
+#'
 #' x <- sort(c(X,as.matrix(seq(from = 0, to = 1, length.out = 101))))
 #' p <- predict(k, x = x, stdev = TRUE, cov = FALSE)
-#' 
+#'
 #' plot(f)
 #' points(X, y)
 #' lines(x, p$mean, col = "blue")
 #' polygon(c(x, rev(x)), c(p$mean - 2 * p$stdev, rev(p$mean + 2 * p$stdev)),
 #' border = NA, col = rgb(0, 0, 1, 0.2))
-#' 
+#'
 #' s <- simulate(k, nsim = 10, seed = 123, x = x)
-#' 
+#'
 #' matlines(x, s, col = rgb(0, 0, 1, 0.2), type = "l", lty = 1)
-NuggetKriging <- function(y, X, kernel,
+NuggetKriging <- function(y=NULL, X=NULL, kernel=NULL,
                     regmodel = c("constant", "linear", "interactive"),
                     normalize = FALSE,
                     optim = c("BFGS", "none"),
@@ -85,7 +84,12 @@ NuggetKriging <- function(y, X, kernel,
     regmodel <- match.arg(regmodel)
     objective <- match.arg(objective)
     if (is.character(optim)) optim <- optim[1] #optim <- match.arg(optim) because we can use BFGS10 for 10 (multistart) BFGS
-    nk <- new_NuggetKriging(y = y, X = X, kernel = kernel,
+    if (is.character(y) && is.null(X) && is.null(kernel)) # just first arg for kernel, without naming
+        nk <- new_NuggetKriging(kernel = y)
+    else if (is.null(y) && is.null(X) && !is.null(kernel))
+        nk <- new_NuggetKriging(kernel = kernel)
+    else
+        nk <- new_NuggetKrigingFit(y = y, X = X, kernel = kernel,
                       regmodel = regmodel,
                       normalize = normalize,
                       optim = optim,
@@ -93,13 +97,10 @@ NuggetKriging <- function(y, X, kernel,
                       parameters = parameters)
     class(nk) <- "NuggetKriging"
     # This will allow to call methods (like in Python/Matlab/Octave) using `k$m(...)` as well as R-style `m(k, ...)`.
-    for (f in methods(class=class(nk))) {
-        if (regexec(paste0(".",class(nk)),f)[[1]]>0) {
-            f_anon = sub(paste0(".",class(nk)),"",fixed=TRUE,f)
-            eval(parse(text=paste0(
-                "nk$", f_anon, " <- function(...) ", f_anon, "(nk,...)"
-                )))
-        }
+    for (f in c('as.km','as.list','copy','fit','logLikelihood','logLikelihoodFun','logMargPost','logMargPostFun','predict','print','show','simulate','update')) {
+        eval(parse(text=paste0(
+            "nk$", f, " <- function(...) ", f, "(nk,...)"
+            )))
     }
     # This will allow to access kriging data/props using `k$d()`
     for (d in c('kernel','optim','objective','X','centerX','scaleX','y','centerY','scaleY','regmodel','F','T','M','z','beta','is_beta_estim','theta','is_theta_estim','sigma2','is_sigma2_estim','nugget','is_nugget_estim')) {
@@ -110,11 +111,11 @@ NuggetKriging <- function(y, X, kernel,
     nk
 }
 
-## ****************************************************************************
+
 #' Coerce a \code{NuggetKriging} Object into a List
 #'
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param x An object with class \code{"NuggetKriging"}.
 #' @param ... Ignored
 #'
@@ -125,7 +126,7 @@ NuggetKriging <- function(y, X, kernel,
 #'     \code{scaleX}, \code{y}, \code{centerY}, \code{scaleY},
 #'     \code{regmodel}, \code{F}, \code{T}, \code{M}, \code{z},
 #'     \code{beta}.
-#' 
+#'
 #' @export
 #' @method as.list NuggetKriging
 #' @aliases as.list,NuggetKriging,NuggetKriging-method
@@ -134,9 +135,9 @@ NuggetKriging <- function(y, X, kernel,
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, kernel = "matern3_2")
-#' 
+#'
 #' l <- as.list(k)
 #' cat(paste0(names(l), " =" , l, collapse = "\n"))
 as.list.NuggetKriging <- function(x, ...) {
@@ -144,23 +145,19 @@ as.list.NuggetKriging <- function(x, ...) {
     nuggetkriging_model(x)
 }
 
-## setMethod("as.list", "NuggetKriging", as.list.NuggetKriging)
 
-## ****************************************************************************
 #' Coerce a \code{NuggetKriging} object into the \code{"km"} class of the
 #' \pkg{DiceKriging} package.
 #'
-#' @title Coerce a \code{NuggetKriging} Object into the Class \code{"km"}
-#' 
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param x An object with S3 class \code{"NuggetKriging"}.
-#' 
+#'
 #' @param .call Force the \code{call} slot to be filled in the
 #'     returned \code{km} object.
 #'
 #' @param ... Not used.
-#' 
+#'
 #' @return An object of having the S4 class \code{"KM"} which extends
 #'     the \code{"km"} class of the \pkg{DiceKriging} package and
 #'     contains an extra \code{NuggetKriging} slot.
@@ -169,35 +166,35 @@ as.list.NuggetKriging <- function(x, ...) {
 #' @importFrom stats model.matrix
 #' @export
 #' @method as.km NuggetKriging
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, "matern3_2")
 #' print(k)
-#' 
+#'
 #' k_km <- as.km(k)
 #' print(k_km)
 as.km.NuggetKriging <- function(x, .call = NULL, ...) {
-    
+
     ## loadDiceKriging()
     ## if (! "DiceKriging" %in% installed.packages())
     ##     stop("DiceKriging must be installed to use its wrapper from libKriging.")
 
     if (!requireNamespace("DiceKriging", quietly = TRUE))
         stop("Package \"DiceKriging\" not found")
-    
+
     model <- new("NuggetKM")
     model@NuggetKriging <- x
-    
+
     if (is.null(.call))
         model@call <- match.call()
     else
         model@call <- .call
-    
+
     m <- nuggetkriging_model(x)
     data <- data.frame(m$X)
     model@trend.formula <- regmodel2formula(m$regmodel)
@@ -211,44 +208,43 @@ as.km.NuggetKriging <- function(x, .call = NULL, ...) {
     model@p <- ncol(m$F)
     model@noise.flag <- FALSE
     model@noise.var <- 0
-    
+
     model@case <- "LLconcentration_beta_v_alpha"
-    
+
     model@known.param <- "None"
     model@param.estim <- NA
     model@method <- m$objective
     model@optim.method <- m$optim
-    
+
     model@penalty <- list()
     model@lower <- 0
     model@upper <- Inf
     model@control <- list()
-    
+
     model@gr <- FALSE
-    
+
     model@T <- t(m$T) * sqrt(m$sigma2)
     model@z <- as.numeric(m$z) / sqrt(m$sigma2)
     model@M <- m$M / sqrt(m$sigma2)
-    
-    covStruct <-  new("covTensorProduct", d = model@d, name = m$kernel, 
-                      sd2 = m$sigma2, var.names = names(data), 
+
+    covStruct <-  new("covTensorProduct", d = model@d, name = m$kernel,
+                      sd2 = m$sigma2, var.names = names(data),
                       nugget = m$nugget, nugget.flag = TRUE, nugget.estim = TRUE,
                       known.covparam = "")
-    
-    covStruct@range.names <- "theta" 
+
+    covStruct@range.names <- "theta"
     covStruct@paramset.n <- as.integer(1)
     covStruct@param.n <- as.integer(model@d)
     covStruct@range.n <- as.integer(model@d)
     covStruct@range.val <- as.numeric(m$theta)
-    model@covariance <- covStruct 
-    
+    model@covariance <- covStruct
+
     return(model)
 }
 
-## ****************************************************************************
+
 #' Print the content of a \code{NuggetKriging} object.
 #'
-#' @title Print a \code{NuggetKriging} object
 #' @author Yann Richet \email{yann.richet@irsn.fr}
 #'
 #' @param x A (S3) \code{NuggetKriging} Object.
@@ -258,30 +254,104 @@ as.km.NuggetKriging <- function(x, .call = NULL, ...) {
 #'
 #' @export
 #' @method print NuggetKriging
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, "matern3_2")
-#' 
+#'
 #' print(k)
 #' ## same thing
 #' k
 print.NuggetKriging <- function(x, ...) {
     if (length(list(...))>0) warning("Arguments ",paste0(names(list(...)),"=",list(...),collapse=",")," are ignored.")
-    k=nuggetkriging_model(x)
-    p = paste0("NuggetKriging model:\n\n",nuggetkriging_summary(x),"\n")
+    p = nuggetkriging_summary(x)
     cat(p)
     invisible(p)
 }
 
-## setMethod("print", "NuggetKriging", print.NuggetKriging)
+
+#' Fit \code{NuggetKriging} object on given data.
+#'
+#' The hyper-parameters (variance and vector of correlation ranges)
+#' are estimated thanks to the optimization of a criterion given by
+#' \code{objective}, using the method given in \code{optim}.
+#'
+#' @author Yann Richet \email{yann.richet@irsn.fr}
+#'
+#' @param object S3 NuggetKriging object.
+#'
+#' @param y Numeric vector of response values.
+#'
+#' @param X Numeric matrix of input design.
+#'
+#' @param regmodel Universal NuggetKriging linear trend.
+#'
+#' @param normalize Logical. If \code{TRUE} both the input matrix
+#'     \code{X} and the response \code{y} in normalized to take
+#'     values in the interval \eqn{[0, 1]}.
+#'
+#' @param optim Character giving the Optimization method used to fit
+#'     hyper-parameters. Possible values are: \code{"BFGS"} and \code{"none"},
+#'     the later simply keeping
+#'     the values given in \code{parameters}. The method
+#'     \code{"BFGS"} uses the gradient of the objective.
+#'
+#' @param objective Character giving the objective function to
+#'     optimize. Possible values are: \code{"LL"} for the
+#'     Log-Likelihood and \code{"LMP"} for the Log-Marginal Posterior.
+#'
+#' @param parameters Initial values for the hyper-parameters. When provided this
+#'     must be named list with some elements \code{"sigma2"}, \code{"theta"}, \code{"nugget"}
+#'     containing the initial value(s) for the variance, range and nugget
+#'     parameters. If \code{theta} is a matrix with more than one row,
+#'     each row is used as a starting point for optimization.
+#'
+#' @param ... Ignored.
+#'
+#' @return No return value. NuggetKriging object argument is modified.
+#'
+#' @method fit NuggetKriging
+#' @export
+#'
+#' @examples
+#' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
+#' plot(f)
+#' set.seed(123)
+#' X <- as.matrix(runif(10))
+#' y <- f(X) + 0.1 * rnorm(nrow(X))
+#' points(X, y, col = "blue", pch = 16)
+#'
+#' k <- NuggetKriging("matern3_2")
+#' print(k)
+#'
+#' fit(k,y,X)
+#' print(k)
+fit.NuggetKriging <- function(object, y, X,
+                    regmodel = c("constant", "linear", "interactive"),
+                    normalize = FALSE,
+                    optim = c("BFGS", "none"),
+                    objective = c("LL", "LMP"),
+                    parameters = NULL, ...) {
+
+    regmodel <- match.arg(regmodel)
+    objective <- match.arg(objective)
+    if (is.character(optim)) optim <- optim[1] #optim <- match.arg(optim) because we can use BFGS10 for 10 (multistart) BFGS
+
+    nuggetkriging_fit(object, y, X,
+                    regmodel,
+                    normalize,
+                    optim ,
+                    objective,
+                    parameters)
+
+    invisible(NULL)
+}
 
 
-## ****************************************************************************
 #' Predict from a \code{NuggetKriging} object.
 #'
 #' Given "new" input points, the method compute the expectation,
@@ -289,12 +359,10 @@ print.NuggetKriging <- function(x, ...) {
 #' stochastic process, conditional on the values at the input points
 #' used when fitting the model.
 #'
-#' @title Prediction Method  for a \code{NuggetKriging} Object
-#' 
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param object S3 NuggetKriging object.
-#' 
+#'
 #' @param x Input points where the prediction must be computed.
 #'
 #' @param stdev \code{Logical}. If \code{TRUE} the standard deviation
@@ -317,10 +385,10 @@ print.NuggetKriging <- function(x, ...) {
 #'     \code{newdata}, \code{stdev} corresponds to \code{se.compute}
 #'     and \code{cov} to \code{cov.compute}. These names are chosen
 #'     \pkg{Python} and \pkg{Octave} interfaces to \pkg{libKriging}.
-#' 
+#'
 #' @method predict NuggetKriging
-#' @export 
-#' 
+#' @export
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' plot(f)
@@ -328,13 +396,13 @@ print.NuggetKriging <- function(x, ...) {
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
 #' points(X, y, col = "blue", pch = 16)
-#' 
+#'
 #' k <- NuggetKriging(y, X, "matern3_2")
-#' 
+#'
 #' ## include design points to see interpolation
 #' x <- sort(c(X,seq(from = 0, to = 1, length.out = 101)))
 #' p <- predict(k, x)
-#' 
+#'
 #' lines(x, p$mean, col = "blue")
 #' polygon(c(x, rev(x)), c(p$mean - 2 * p$stdev, rev(p$mean + 2 * p$stdev)),
 #'  border = NA, col = rgb(0, 0, 1, 0.2))
@@ -350,20 +418,14 @@ predict.NuggetKriging <- function(object, x, stdev = TRUE, cov = FALSE, deriv =
     return(nuggetkriging_predict(object, x, stdev, cov, deriv))
 }
 
-## predict <- function (...) UseMethod("predict")
-## setMethod("predict", "NuggetKriging", predict.NuggetKriging)
-
-## ****************************************************************************
 #' Simulation from a \code{NuggetKriging} model object.
 #'
 #' This method draws paths of the stochastic process at new input
 #' points conditional on the values at the input points used in the
 #' fit.
 #'
-#' @title Simulation from a \code{NuggetKriging} Object
-#' 
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param object S3 NuggetKriging object.
 #' @param nsim Number of simulations to perform.
 #' @param seed Random seed used.
@@ -379,11 +441,11 @@ predict.NuggetKriging <- function(object, x, stdev = TRUE, cov = FALSE, deriv =
 #'     \code{"KM"}. The formal \code{x} corresponds to
 #'     \code{newdata}. These names are chosen \pkg{Python} and
 #'     \pkg{Octave} interfaces to \pkg{libKriging}.
-#' 
-#' 
+#'
+#'
 #' @method simulate NuggetKriging
 #' @export
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' plot(f)
@@ -391,18 +453,18 @@ predict.NuggetKriging <- function(object, x, stdev = TRUE, cov = FALSE, deriv =
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1  *rnorm(nrow(X))
 #' points(X, y, col = "blue")
-#' 
+#'
 #' k <- NuggetKriging(y, X, kernel = "matern3_2")
-#' 
+#'
 #' x <- seq(from = 0, to = 1, length.out = 101)
 #' s <- simulate(k, nsim = 3, x = x)
-#' 
+#'
 #' lines(x, s[ , 1], col = "blue")
 #' lines(x, s[ , 2], col = "blue")
 #' lines(x, s[ , 3], col = "blue")
 simulate.NuggetKriging <- function(object, nsim = 1, seed = 123, x,  ...) {
     if (length(L <- list(...)) > 0) warnOnDots(L)
-    k <- nuggetkriging_model(object) 
+    k <- nuggetkriging_model(object)
     if (is.data.frame(x)) x = data.matrix(x)
     if (!is.matrix(x)) x = matrix(x, ncol = ncol(k$X))
     if (ncol(x) != ncol(k$X))
@@ -413,38 +475,31 @@ simulate.NuggetKriging <- function(object, nsim = 1, seed = 123, x,  ...) {
     return(nuggetkriging_simulate(object, nsim = nsim, seed = seed, X = x))
 }
 
-## simulate <- function (...) UseMethod("simulate")
-## setMethod("simulate", "NuggetKriging", simulate.NuggetKriging)
 
-
-## removed by Yves @aliases update,NuggetKriging,NuggetKriging-method
-## *****************************************************************************
 #' Update a \code{NuggetKriging} model object with new points
-#' 
-#' @title Update a \code{NuggetKriging} Object with New Points
+#'
 #' @author Yann Richet \email{yann.richet@irsn.fr}
 #'
-#' 
 #' @param object S3 NuggetKriging object.
 #'
 #' @param newy Numeric vector of new responses (output).
 #'
 #' @param newX Numeric matrix of new input points.
-#' 
+#'
 #' @param ... Ignored.
-#' 
+#'
 #' @return No return value. NuggetKriging object argument is modified.
-#' 
+#'
 #' @section Caution: The method \emph{does not return the updated
 #'     object}, but instead changes the content of
 #'     \code{object}. This behaviour is quite unusual in R and
 #'     differs from the behaviour of the methods
 #'     \code{\link[DiceKriging]{update.km}} in \pkg{DiceKriging} and
 #'     \code{\link{update,KM-method}}.
-#'  
+#'
 #' @method update NuggetKriging
-#' @export 
-#' 
+#' @export
+#'
 #' @examples
 #' f <- function(x) 1- 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x)*x^5 + 0.7)
 #' plot(f)
@@ -452,23 +507,23 @@ simulate.NuggetKriging <- function(object, nsim = 1, seed = 123, x,  ...) {
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
 #' points(X, y, col = "blue")
-#' 
+#'
 #' k <- NuggetKriging(y, X, "matern3_2")
-#' 
+#'
 #' ## include design points to see interpolation
 #' x <- sort(c(X,seq(from = 0, to = 1, length.out = 101)))
 #' p <- predict(k, x)
 #' lines(x, p$mean, col = "blue")
 #' polygon(c(x, rev(x)), c(p$mean - 2 * p$stdev, rev(p$mean + 2 * p$stdev)),
 #'  border = NA, col = rgb(0, 0, 1, 0.2))
-#' 
+#'
 #' newX <- as.matrix(runif(3))
 #' newy <- f(newX) + 0.1 * rnorm(nrow(newX))
 #' points(newX, newy, col = "red")
-#' 
+#'
 #' ## change the content of the object 'k'
 #' update(k, newy, newX)
-#' 
+#'
 #' ## include design points to see interpolation
 #' x <- sort(c(X,newX,seq(from = 0, to = 1, length.out = 101)))
 #' p2 <- predict(k, x)
@@ -476,9 +531,9 @@ simulate.NuggetKriging <- function(object, nsim = 1, seed = 123, x,  ...) {
 #' polygon(c(x, rev(x)), c(p2$mean - 2 * p2$stdev, rev(p2$mean + 2 * p2$stdev)),
 #'  border = NA, col = rgb(1, 0, 0, 0.2))
 update.NuggetKriging <- function(object, newy, newX, ...) {
-    
+
     if (length(L <- list(...)) > 0) warnOnDots(L)
-    k <- nuggetkriging_model(object) 
+    k <- nuggetkriging_model(object)
     if (is.data.frame(newX)) newX = data.matrix(newX)
     if (!is.matrix(newX)) newX <- matrix(newX, ncol = ncol(k$X))
     if (is.data.frame(newy)) newy = data.matrix(newy)
@@ -491,61 +546,129 @@ update.NuggetKriging <- function(object, newy, newX, ...) {
 
     ## Modify 'object' in the parent environment
     nuggetkriging_update(object, newy, newX)
-    
+
     invisible(NULL)
 }
 
-## update <- function(...) UseMethod("update")
-## setMethod("update", "NuggetKriging", update.NuggetKriging)
-## setGeneric(name = "update", def = function(...) standardGeneric("update"))
 
-## ****************************************************************************
-#' Compute Log-Likelihood of NuggetKriging Model
-#' 
+#' Save a NuggetKriging Model to a file storage
+#'
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
+#' @param object An S3 NuggetKriging object.
+#' @param filename File name to save in.
+#' @param ... Not used.
+#'
+#' @return The loaded NuggetKriging object.
+#'
+#' @method save NuggetKriging
+#' @export
+#' @aliases save,NuggetKriging,NuggetKriging-method
+#'
+#' @examples
+#' f <- function(x) 1- 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x)*x^5 + 0.7)
+#' set.seed(123)
+#' X <- as.matrix(runif(10))
+#' y <- f(X) + 0.1 * rnorm(nrow(X))
+#' points(X, y, col = "blue")
+#'
+#' k <- NuggetKriging(y, X, "matern3_2")
+#' print(k)
+#'
+#' outfile = tempfile("k.h5") 
+#' save(k,outfile)
+save.NuggetKriging <- function(object, filename, ...) {
+
+    if (length(L <- list(...)) > 0) warnOnDots(L)
+    if (!is.character(filename))
+        stop("'filename' must be a string")
+
+    nuggetkriging_save(object, filename)
+
+    invisible(NULL)
+}
+
+
+#' Load a NuggetKriging Model from a file storage
+#'
+#' @author Yann Richet \email{yann.richet@irsn.fr}
+#'
+#' @param filename File name to load from.
+#' @param ... Not used.
+#'
+#' @return The loaded NuggetKriging object.
+#'
+#' @export
+#'
+#' @examples
+#' f <- function(x) 1- 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x)*x^5 + 0.7)
+#' set.seed(123)
+#' X <- as.matrix(runif(10))
+#' y <- f(X) + 0.1 * rnorm(nrow(X))
+#' points(X, y, col = "blue")
+#'
+#' k <- NuggetKriging(y, X, "matern3_2")
+#' print(k)
+#'
+#' outfile = tempfile("k.h5")
+#' save(k,outfile)
+#'
+#' print(load.NuggetKriging(outfile)) 
+load.NuggetKriging <- function(filename, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
+    if (!is.character(filename))
+        stop("'filename' must be a string")
+    return( nuggetkriging_load(filename) )
+}
+
+
+#' Compute Log-Likelihood of NuggetKriging Model
+#'
+#' @author Yann Richet \email{yann.richet@irsn.fr}
+#'
 #' @param object An S3 NuggetKriging object.
 #' @param theta_alpha A numeric vector of (positive) range parameters and variance over variance plus nugget at
 #'     which the log-likelihood will be evaluated.
 #' @param grad Logical. Should the function return the gradient?
+#' @param bench Logical. Should the function display benchmarking output
 #' @param ... Not used.
-#' 
+#'
 #' @return The log-Likelihood computed for given
 #'     \eqn{\boldsymbol{theta_alpha}}{\frac{\sigma^2}{\sigma^2+nugget}}.
-#' 
+#'
 #' @method logLikelihoodFun NuggetKriging
-#' @export 
+#' @export
 #' @aliases logLikelihoodFun,NuggetKriging,NuggetKriging-method
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, kernel = "matern3_2")
 #' print(k)
-#' 
+#'
 #' theta0 = k$theta()
 #' ll_alpha <- function(alpha) logLikelihoodFun(k,cbind(theta0,alpha))$logLikelihood
 #' a <- seq(from = 0.9, to = 1.0, length.out = 101)
 #' plot(a, Vectorize(ll_alpha)(a), type = "l",xlim=c(0.9,1))
 #' abline(v = k$sigma2()/(k$sigma2()+k$nugget()), col = "blue")
-#' 
+#'
 #' alpha0 = k$sigma2()/(k$sigma2()+k$nugget())
 #' ll_theta <- function(theta) logLikelihoodFun(k,cbind(theta,alpha0))$logLikelihood
 #' t <- seq(from = 0.001, to = 2, length.out = 101)
 #' plot(t, Vectorize(ll_theta)(t), type = 'l')
 #' abline(v = k$theta(), col = "blue")
-#' 
+#'
 #' ll <- function(theta_alpha) logLikelihoodFun(k,theta_alpha)$logLikelihood
 #' a <- seq(from = 0.9, to = 1.0, length.out = 31)
 #' t <- seq(from = 0.001, to = 2, length.out = 101)
 #' contour(t,a,matrix(ncol=length(a),ll(expand.grid(t,a))),xlab="theta",ylab="sigma2/(sigma2+nugget)")
 #' points(k$theta(),k$sigma2()/(k$sigma2()+k$nugget()),col='blue')
 logLikelihoodFun.NuggetKriging <- function(object, theta_alpha,
-                                  grad = FALSE, ...) {
-    k <- nuggetkriging_model(object) 
+                                  grad = FALSE, bench=FALSE, ...) {
+    k <- nuggetkriging_model(object)
     if (is.data.frame(theta_alpha)) theta_alpha = data.matrix(theta_alpha)
     if (!is.matrix(theta_alpha)) theta_alpha <- matrix(theta_alpha, ncol = ncol(k$X)+1)
     if (ncol(theta_alpha) != ncol(k$X)+1)
@@ -556,100 +679,98 @@ logLikelihoodFun.NuggetKriging <- function(object, theta_alpha,
                                            ncol = ncol(theta_alpha)))
     for (i in 1:nrow(theta_alpha)) {
         ll <- nuggetkriging_logLikelihoodFun(object, theta_alpha[i, ],
-                                    grad = isTRUE(grad))
+                                    grad = isTRUE(grad), bench = isTRUE(bench))
         out$logLikelihood[i] <- ll$logLikelihood
         if (isTRUE(grad)) out$logLikelihoodGrad[i, ] <- ll$logLikelihoodGrad
     }
     if (!isTRUE(grad)) out$logLikelihoodGrad <- NULL
-  
+
     return(out)
 }
 
 
-## ****************************************************************************
+
 #' Get logLikelihood of NuggetKriging Model
-#' 
+#'
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param object An S3 NuggetKriging object.
 #' @param ... Not used.
-#' 
+#'
 #' @return The logLikelihood computed for fitted
 #'     \eqn{\boldsymbol{theta_alpha}}{\theta,\frac{\sigma^2}{\sigma^2+nugget}}.
-#' 
+#'
 #' @method logLikelihood NuggetKriging
-#' @export 
+#' @export
 #' @aliases logLikelihood,NuggetKriging,NuggetKriging-method
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, kernel = "matern3_2", objective="LL")
 #' print(k)
-#' 
+#'
 #' logLikelihood(k)
 logLikelihood.NuggetKriging <- function(object, ...) {
   return(nuggetkriging_logLikelihood(object))
 }
 
-##****************************************************************************
 #' Compute the log-marginal posterior of a kriging model, using the
 #' prior XXXY.
 #'
-#' @title Compute Log-Marginal-Posterior of NuggetKriging Model
-#' 
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param object S3 NuggetKriging object.
 #' @param theta_alpha Numeric vector of correlation range and variance over variance plus nugget parameters at
 #'     which the function is to be evaluated.
 #' @param grad Logical. Should the function return the gradient
 #'     (w.r.t theta_alpha)?
+#' @param bench Logical. Should the function display benchmarking output
 #' @param ... Not used.
-#' 
+#'
 #' @return The value of the log-marginal posterior computed for the
 #'     given vector \eqn{\boldsymbol{theta_alpha}}{\theta,\frac{\sigma^2}{\sigma^2+nugget}}.
-#' 
+#'
 #' @method logMargPostFun NuggetKriging
-#' @export 
+#' @export
 #' @aliases logMargPostFun,NuggetKriging,NuggetKriging-method
 #'
 #' @references
 #' XXXY A reference describing the model (prior, ...)
 #'
 #' @seealso \code{\link[RobustGaSP]{rgasp}} in the RobustGaSP package.
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, "matern3_2", objective="LMP")
 #' print(k)
-#' 
+#'
 #' theta0 = k$theta()
 #' lmp_alpha <- function(alpha) k$logMargPostFun(cbind(theta0,alpha))$logMargPost
 #' a <- seq(from = 0.9, to = 1.0, length.out = 101)
 #' plot(a, Vectorize(lmp_alpha)(a), type = "l",xlim=c(0.9,1))
 #' abline(v = k$sigma2()/(k$sigma2()+k$nugget()), col = "blue")
-#' 
+#'
 #' alpha0 = k$sigma2()/(k$sigma2()+k$nugget())
 #' lmp_theta <- function(theta) k$logMargPostFun(cbind(theta,alpha0))$logMargPost
 #' t <- seq(from = 0.001, to = 2, length.out = 101)
 #' plot(t, Vectorize(lmp_theta)(t), type = 'l')
 #' abline(v = k$theta(), col = "blue")
-#' 
+#'
 #' lmp <- function(theta_alpha) k$logMargPostFun(theta_alpha)$logMargPost
 #' t <- seq(from = 0.4, to = 0.6, length.out = 51)
 #' a <- seq(from = 0.9, to = 1, length.out = 51)
 #' contour(t,a,matrix(ncol=length(t),lmp(expand.grid(t,a))),
 #'  nlevels=50,xlab="theta",ylab="sigma2/(sigma2+nugget)")
 #' points(k$theta(),k$sigma2()/(k$sigma2()+k$nugget()),col='blue')
-logMargPostFun.NuggetKriging <- function(object, theta_alpha, grad = FALSE, ...) {
+logMargPostFun.NuggetKriging <- function(object, theta_alpha, grad = FALSE, bench=FALSE, ...) {
     k <- nuggetkriging_model(object)
     if (is.data.frame(theta_alpha)) theta_alpha = data.matrix(theta_alpha)
     if (!is.matrix(theta_alpha)) theta_alpha <- matrix(theta_alpha,ncol=ncol(k$X)+1)
@@ -660,7 +781,7 @@ logMargPostFun.NuggetKriging <- function(object, theta_alpha, grad = FALSE, ...)
                 logMargPostGrad = matrix(NA, nrow = nrow(theta_alpha),
                                          ncol = ncol(theta_alpha)))
     for (i in 1:nrow(theta_alpha)) {
-        lmp <- nuggetkriging_logMargPostFun(object, theta_alpha[i, ], grad = isTRUE(grad))
+        lmp <- nuggetkriging_logMargPostFun(object, theta_alpha[i, ], grad = isTRUE(grad), bench = isTRUE(bench))
         out$logMargPost[i] <- lmp$logMargPost
         if (isTRUE(grad)) out$logMargPostGrad[i, ] <- lmp$logMargPostGrad
     }
@@ -668,31 +789,59 @@ logMargPostFun.NuggetKriging <- function(object, theta_alpha, grad = FALSE, ...)
     return(out)
 }
 
-## ****************************************************************************
+
 #' Get logMargPost of NuggetKriging Model
-#' 
+#'
 #' @author Yann Richet \email{yann.richet@irsn.fr}
-#' 
+#'
 #' @param object An S3 NuggetKriging object.
 #' @param ... Not used.
-#' 
+#'
 #' @return The logMargPost computed for fitted
 #'     \eqn{\boldsymbol{theta_alpha}}{\theta,\frac{\sigma^2}{\sigma^2+nugget}}.
-#' 
+#'
 #' @method logMargPost NuggetKriging
-#' @export 
+#' @export
 #' @aliases logMargPost,NuggetKriging,NuggetKriging-method
-#' 
+#'
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
 #' X <- as.matrix(runif(10))
 #' y <- f(X) + 0.1 * rnorm(nrow(X))
-#' 
+#'
 #' k <- NuggetKriging(y, X, kernel = "matern3_2", objective="LMP")
 #' print(k)
-#' 
+#'
 #' logMargPost(k)
 logMargPost.NuggetKriging <- function(object, ...) {
   return(nuggetkriging_logMargPost(object))
 }
+
+
+#' Duplicate a NuggetKriging Model
+#'
+#' @author Yann Richet \email{yann.richet@irsn.fr}
+#'
+#' @param object An S3 NuggetKriging object.
+#' @param ... Not used.
+#'
+#' @return The copy of object.
+#'
+#' @method copy NuggetKriging
+#' @export
+#' @aliases copy,NuggetKriging,NuggetKriging-method
+#'
+#' @examples
+#' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
+#' set.seed(123)
+#' X <- as.matrix(runif(10))
+#' y <- f(X) + 0.1 * rnorm(nrow(X))
+#'
+#' k <- NuggetKriging(y, X, kernel = "matern3_2", objective="LMP")
+#' print(k)
+#'
+#' print(copy(k))
+copy.NuggetKriging <- function(object, ...) {
+  return(nuggetkriging_copy(object))
+}
