@@ -43,8 +43,9 @@ Rcpp::List new_NoiseKrigingFit(arma::vec y,
     if (params.containsElementNamed("sigma2")) {
       _parameters.push_back(params["sigma2"], "sigma2");
       _parameters.push_back(true, "has_sigma2");
-      _parameters.push_back(!(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]),
-                            "is_sigma2_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]) && optim != "none",
+          "is_sigma2_estim");
     } else {
       //_parameters.push_back(Rcpp::runif(1), "sigma2"); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(true, "has_sigma2");
@@ -55,8 +56,9 @@ Rcpp::List new_NoiseKrigingFit(arma::vec y,
     if (params.containsElementNamed("theta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["theta"]), "theta");
       _parameters.push_back(true, "has_theta");
-      _parameters.push_back(!(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]),
-                            "is_theta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]) && optim != "none",
+          "is_theta_estim");
     } else {
       // Rcpp::NumericVector r = Rcpp::runif(X.n_cols); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(Rcpp::NumericMatrix(1, X.n_cols,
@@ -68,8 +70,9 @@ Rcpp::List new_NoiseKrigingFit(arma::vec y,
     if (params.containsElementNamed("beta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["beta"]), "beta");
       _parameters.push_back(true, "has_beta");
-      _parameters.push_back(!(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]),
-                            "is_beta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]) && optim != "none",
+          "is_beta_estim");
     } else {
       _parameters.push_back(Rcpp::NumericVector(0), "beta");
       _parameters.push_back(false, "has_beta");
@@ -141,8 +144,9 @@ void noisekriging_fit(Rcpp::List k,
     if (params.containsElementNamed("sigma2")) {
       _parameters.push_back(params["sigma2"], "sigma2");
       _parameters.push_back(true, "has_sigma2");
-      _parameters.push_back(!(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]),
-                            "is_sigma2_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]) && optim != "none",
+          "is_sigma2_estim");
     } else {
       //_parameters.push_back(Rcpp::runif(1), "sigma2"); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(true, "has_sigma2");
@@ -153,8 +157,9 @@ void noisekriging_fit(Rcpp::List k,
     if (params.containsElementNamed("theta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["theta"]), "theta");
       _parameters.push_back(true, "has_theta");
-      _parameters.push_back(!(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]),
-                            "is_theta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]) && optim != "none",
+          "is_theta_estim");
     } else {
       // Rcpp::NumericVector r = Rcpp::runif(X.n_cols); // turnaround mingw bug:
       // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(Rcpp::NumericMatrix(1, X.n_cols,
@@ -166,8 +171,9 @@ void noisekriging_fit(Rcpp::List k,
     if (params.containsElementNamed("beta")) {
       _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["beta"]), "beta");
       _parameters.push_back(true, "has_beta");
-      _parameters.push_back(!(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]),
-                            "is_beta_estim");
+      _parameters.push_back(
+          !(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]) && optim != "none",
+          "is_beta_estim");
     } else {
       _parameters.push_back(Rcpp::NumericVector(0), "beta");
       _parameters.push_back(false, "has_beta");
@@ -321,14 +327,25 @@ void noisekriging_update(Rcpp::List k, arma::vec y, arma::vec noise, arma::mat X
 }
 
 // [[Rcpp::export]]
-Rcpp::List noisekriging_logLikelihoodFun(Rcpp::List k, arma::vec theta_sigma2, bool grad = false) {
+void noisekriging_save(Rcpp::List k, std::string filename) {
   if (!k.inherits("NoiseKriging"))
     Rcpp::stop("Input must be a NoiseKriging object.");
   SEXP impl = k.attr("object");
 
   Rcpp::XPtr<NoiseKriging> impl_ptr(impl);
 
-  std::tuple<double, arma::vec> ll = impl_ptr->logLikelihoodFun(theta_sigma2, grad);
+  return impl_ptr->save(filename);
+}
+
+// [[Rcpp::export]]
+Rcpp::List noisekriging_logLikelihoodFun(Rcpp::List k, arma::vec theta_sigma2, bool grad = false, bool bench = false) {
+  if (!k.inherits("NoiseKriging"))
+    Rcpp::stop("Input must be a NoiseKriging object.");
+  SEXP impl = k.attr("object");
+
+  Rcpp::XPtr<NoiseKriging> impl_ptr(impl);
+
+  std::tuple<double, arma::vec> ll = impl_ptr->logLikelihoodFun(theta_sigma2, grad, bench);
 
   Rcpp::List ret = Rcpp::List::create(Rcpp::Named("logLikelihood") = std::get<0>(ll));
   if (grad) {
