@@ -1,7 +1,3 @@
-library(testthat)
-#library(rlibkriging, lib.loc="bindings/R/Rlibs")
-
-
 context("Check predict args (T,F) are consistent")
 
 f = function(x) 1-1/2*(sin(12*x)/(1+x)+2*cos(7*x)*x^5+0.7)
