@@ -6,6 +6,7 @@
 
 #include "libKriging/utils/lk_armadillo.hpp"
 
+#include "libKriging/Bench.hpp"
 #include "libKriging/CacheFunction.hpp"
 #include "libKriging/Covariance.hpp"
 #include "libKriging/KrigingException.hpp"
@@ -16,11 +17,14 @@
 #include "libKriging/Trend.hpp"
 #include "libKriging/utils/custom_hash_function.hpp"
 #include "libKriging/utils/data_from_arma_vec.hpp"
+#include "libKriging/utils/hdf5utils.hpp"
+#include "libKriging/utils/utils.hpp"
 
 #include <cassert>
 #include <lbfgsb_cpp/lbfgsb.hpp>
 #include <tuple>
 #include <vector>
+#include <map>
 
 /************************************************/
 /**      NuggetKriging implementation        **/
@@ -95,7 +99,8 @@ LIBKRIGING_EXPORT NuggetKriging::NuggetKriging(const NuggetKriging& other, Expli
 
 double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
                                      arma::vec* grad_out,
-                                     NuggetKriging::OKModel* okm_data) const {
+                                     NuggetKriging::OKModel* okm_data,
+                                     std::map<std::string, double>* bench) const {
   // arma::cout << " theta, alpha: " << _theta_alpha.t() << arma::endl;
   //' @ref https://github.com/cran/DiceKriging/blob/master/R/logLikFun.R
   //   model@covariance <- vect2covparam(model@covariance, param[1:(nparam - 1)])
@@ -121,6 +126,7 @@ double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
   double _alpha = _theta_alpha.at(d);
   arma::vec _theta = _theta_alpha.head(d);
 
+  auto t0 = Bench::tic();
   arma::mat R = arma::mat(n, n);
   for (arma::uword i = 0; i < n; i++) {
     R.at(i, i) = 1;
@@ -128,32 +134,42 @@ double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
       R.at(i, j) = R.at(j, i) = Cov(m_dX.col(i * n + j), _theta) * _alpha;
     }
   }
+  t0 = Bench::toc(bench, "R = Cov(dX)", t0);
 
   // Cholesky decompostion of covariance matrix
   fd->T = LinearAlgebra::safe_chol_lower(R);  // Do NOT trimatl T (slower because copy): trimatl(chol(R, "lower"));
+  t0 = Bench::toc(bench, "T = Chol(R)", t0);
 
   // Compute intermediate useful matrices
   fd->M = solve(fd->T, m_F, LinearAlgebra::default_solve_opts);
-  arma::mat Q;
-  arma::mat G;
-  qr_econ(Q, G, fd->M);
+  t0 = Bench::toc(bench, "M = F \\ T", t0);
 
   arma::colvec Yt = solve(fd->T, m_y, LinearAlgebra::default_solve_opts);
-  if (fd->is_beta_estim)
-    fd->beta = solve(G, Q.t() * Yt, LinearAlgebra::default_solve_opts);
-  fd->z = Yt - fd->M * fd->beta;
+  t0 = Bench::toc(bench, "Yt = y \\ T", t0);
 
-  fd->var = arma::accu(fd->z % fd->z) / n;
-  if (fd->is_nugget_estim) {
-    fd->nugget = (1 - _alpha) * fd->var;
-    if (fd->is_sigma2_estim)
-      fd->sigma2 = _alpha * fd->var;
-  } else {
-    if (fd->is_sigma2_estim)
-      fd->sigma2 = fd->var - fd->nugget;
+  if (fd->is_beta_estim) {
+    arma::mat Q;
+    arma::mat G;
+    qr_econ(Q, G, fd->M);
+    t0 = Bench::toc(bench, "Q,G = QR(M)", t0);
+    fd->beta = solve(G, Q.t() * Yt, LinearAlgebra::default_solve_opts);
+    t0 = Bench::toc(bench, "B = Qt * Yt \\ G", t0);
   }
 
+  fd->z = Yt - fd->M * fd->beta;
+  t0 = Bench::toc(bench, "z = Yt - M * B", t0);
+
+  fd->var = arma::accu(fd->z % fd->z) / n;
+  t0 = Bench::toc(bench, "S2 = Acc(z * z) / n", t0);
+
+  if (fd->is_nugget_estim)
+    fd->nugget = (1 - _alpha) * fd->var;
+
+  if (fd->is_sigma2_estim)
+    fd->sigma2 = _alpha * fd->var;
+
   double ll = -0.5 * (n * log(2 * M_PI * fd->var) + 2 * sum(log(fd->T.diag())) + n);
+  t0 = Bench::toc(bench, "ll = ...log(S2) + Sum(log(Td))...", t0);
   // arma::cout << " ll:" << ll << arma::endl;
 
   if (grad_out != nullptr) {
@@ -179,15 +195,20 @@ double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
     //      logLik.derivative[k] <- term1 + term2
     //  }
 
+    t0 = Bench::tic();
     std::vector<arma::mat> gradsC(d);  // if (hess_out != nullptr)
     arma::vec term1 = arma::vec(d);    // if (hess_out != nullptr)
 
     arma::mat Linv = solve(fd->T, arma::eye(n, n), LinearAlgebra::default_solve_opts);
+    t0 = Bench::toc(bench, "Li = I \\ T", t0);
     arma::mat Cinv = (Linv.t() * Linv);  // Do NOT inv_sympd (slower): inv_sympd(R);
+    t0 = Bench::toc(bench, "Ri = Lit * Li", t0);
 
     arma::mat tT = fd->T.t();  // trimatu(trans(fd->T));
+    t0 = Bench::toc(bench, "tT = Tt", t0);
+
     arma::mat x = solve(tT, fd->z, LinearAlgebra::default_solve_opts);
-    arma::mat xx = x * x.t();
+    t0 = Bench::toc(bench, "x = z \\ tT", t0);
 
     arma::cube gradC = arma::cube(d, n, n);
     for (arma::uword i = 0; i < n; i++) {
@@ -195,8 +216,10 @@ double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
         gradC.slice(i).col(j) = R.at(i, j) * DlnCovDtheta(m_dX.col(i * n + j), _theta);
       }
     }
+    t0 = Bench::toc(bench, "gradR = R * dlnCov(dX)", t0);
 
     for (arma::uword k = 0; k < d; k++) {
+      t0 = Bench::tic();
       arma::mat gradC_k = arma::mat(n, n);
       for (arma::uword i = 0; i < n; i++) {
         gradC_k.at(i, i) = 0;
@@ -204,12 +227,14 @@ double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
           gradC_k.at(i, j) = gradC_k.at(j, i) = gradC.slice(i).col(j)[k];
         }
       }
+      t0 = Bench::toc(bench, "gradR_k = gradR[k]", t0);
 
       // should make a fast function trace_prod(A,B) -> sum_i(sum_j(Ai,j*Bj,i))
       term1.at(k)
           = as_scalar((trans(x) * gradC_k) * x) / fd->var;  //; //as_scalar((trans(x) * gradR_k) * x)/ sigma2_hat;
       double term2 = -arma::trace(Cinv * gradC_k);          //-arma::accu(Rinv % gradR_k_upper)
       (*grad_out).at(k) = (term1.at(k) + term2) / 2;
+      t0 = Bench::toc(bench, "grad_ll[k] = xt * gradR_k / S2 + tr(Ri * gradR_k)", t0);
     }  // for (arma::uword k = 0; k < m_X.n_cols; k++)
 
     //  # partial derivative with respect to v = sigma^2 + delta^2
@@ -229,7 +254,8 @@ double NuggetKriging::_logLikelihood(const arma::vec& _theta_alpha,
 }
 
 LIBKRIGING_EXPORT std::tuple<double, arma::vec> NuggetKriging::logLikelihoodFun(const arma::vec& _theta_alpha,
-                                                                                const bool _grad) {
+                                                                                const bool _grad,
+                                                                                const bool _bench) {
   arma::mat T;
   arma::mat M;
   arma::colvec z;
@@ -241,11 +267,28 @@ LIBKRIGING_EXPORT std::tuple<double, arma::vec> NuggetKriging::logLikelihoodFun(
 
   double ll = -1;
   arma::vec grad;
-  if (_grad) {
-    grad = arma::vec(_theta_alpha.n_elem);
-    ll = _logLikelihood(_theta_alpha, &grad, &okm_data);
-  } else
-    ll = _logLikelihood(_theta_alpha, nullptr, &okm_data);
+
+  if (_bench) {
+    std::map<std::string, double> bench;
+    if (_grad) {
+      grad = arma::vec(_theta_alpha.n_elem);
+      ll = _logLikelihood(_theta_alpha, &grad, &okm_data, &bench);
+    } else
+      ll = _logLikelihood(_theta_alpha, nullptr, &okm_data, &bench);
+
+    size_t num = 0;
+    for (auto& kv : bench)
+      num = std::max(kv.first.size(), num);
+    for (auto& kv : bench)
+      arma::cout << "| " << Bench::pad(kv.first, num, ' ') << " | " << kv.second << " |" << arma::endl;
+
+  } else {
+    if (_grad) {
+      grad = arma::vec(_theta_alpha.n_elem);
+      ll = _logLikelihood(_theta_alpha, &grad, &okm_data, nullptr);
+    } else
+      ll = _logLikelihood(_theta_alpha, nullptr, &okm_data, nullptr);
+  }
 
   return std::make_tuple(ll, std::move(grad));
 }
@@ -253,7 +296,8 @@ LIBKRIGING_EXPORT std::tuple<double, arma::vec> NuggetKriging::logLikelihoodFun(
 
 double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
                                    arma::vec* grad_out,
-                                   NuggetKriging::OKModel* okm_data) const {
+                                   NuggetKriging::OKModel* okm_data,
+                                   std::map<std::string, double>* bench) const {
   // arma::cout << " theta: " << _theta << arma::endl;
 
   // In RobustGaSP:
@@ -309,6 +353,7 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
   double _alpha = _theta_alpha.at(d);
   arma::vec _theta = _theta_alpha.head(d);
 
+  auto t0 = Bench::tic();
   arma::mat R = arma::mat(n, n);
   for (arma::uword i = 0; i < n; i++) {
     R.at(i, i) = 1;
@@ -316,9 +361,11 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
       R.at(i, j) = R.at(j, i) = Cov(m_dX.col(i * n + j), _theta) * _alpha;
     }
   }
+  t0 = Bench::toc(bench, "R = Cov(dX)", t0);
 
   // Cholesky decompostion of covariance matrix
   fd->T = LinearAlgebra::safe_chol_lower(R);
+  t0 = Bench::toc(bench, "T = Chol(R)", t0);
 
   //  // Compute intermediate useful matrices
   //  fd->M = solve(fd->T, m_F, LinearAlgebra::solve_opts);
@@ -337,49 +384,70 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
   arma::mat L = fd->T;
 
   fd->M = solve(L, X, LinearAlgebra::default_solve_opts);
+  t0 = Bench::toc(bench, "M = F \\ T", t0);
+
   arma::mat R_inv_X = solve(trans(L), fd->M, LinearAlgebra::default_solve_opts);
+  t0 = Bench::toc(bench, "RiF = Ri * F", t0);
+
   arma::mat Xt_R_inv_X = trans(X) * R_inv_X;  // Xt%*%R.inv%*%X
+  t0 = Bench::toc(bench, "FtRiF = Ft * RiF", t0);
 
   arma::mat LX = chol(Xt_R_inv_X, "lower");  //  retrieve factor LX  in the decomposition
+  t0 = Bench::toc(bench, "TF = Chol(FtRiF)", t0);
+
   arma::mat R_inv_X_Xt_R_inv_X_inv_Xt_R_inv
       = R_inv_X
         * (solve(trans(LX),
                  solve(LX, trans(R_inv_X), LinearAlgebra::default_solve_opts),
                  LinearAlgebra::default_solve_opts));  // compute  R_inv_X_Xt_R_inv_X_inv_Xt_R_inv through one forward
                                                        // and one backward solve
+  t0 = Bench::toc(bench, "RiFFtRiFiFtRi = RiF * RiFt \\ M \\ Mt", t0);
+
   arma::colvec Yt = solve(L, m_y, LinearAlgebra::default_solve_opts);
+  t0 = Bench::toc(bench, "Yt = y \\ T", t0);
+
   if (fd->is_beta_estim) {
     arma::mat Q;
     arma::mat G;
     qr_econ(Q, G, fd->M);
+    t0 = Bench::toc(bench, "Q,G = QR(M)", t0);
     fd->beta = solve(G, Q.t() * Yt, LinearAlgebra::default_solve_opts);
-    fd->z = Yt - fd->M * fd->beta;
+    t0 = Bench::toc(bench, "B = Qt * Yt \\ G", t0);
   }
 
+  fd->z = Yt - fd->M * fd->beta;  // required for later predict
+  t0 = Bench::toc(bench, "z = Yt - M * B", t0);
+
   arma::mat yt_R_inv = trans(solve(trans(L), Yt, LinearAlgebra::default_solve_opts));
-  arma::mat S_2 = (yt_R_inv * m_y - trans(m_y) * R_inv_X_Xt_R_inv_X_inv_Xt_R_inv * m_y);
+  t0 = Bench::toc(bench, "YtRi = Yt \\ Tt", t0);
 
-  fd->var = S_2(0, 0) / (n - d);
-  if (fd->is_nugget_estim) {
-    fd->nugget = (1 - _alpha) * fd->var;
-    if (fd->is_sigma2_estim)
-      fd->sigma2 = _alpha * fd->var;
-  } else {
-    if (fd->is_sigma2_estim)
-      fd->sigma2 = fd->var - fd->nugget;
-  }
+  arma::mat S_2 = (yt_R_inv * m_y - trans(m_y) * R_inv_X_Xt_R_inv_X_inv_Xt_R_inv * m_y);
+  t0 = Bench::toc(bench, "S2 = YtRi * y - yt * RiFFtRiFiFtRi * y", t0);
+  // arma::cout << " S_2:" << S_2 << arma::endl;
+
+  if (fd->is_sigma2_estim)
+    fd->sigma2 = S_2(0, 0) / (n - d);
+
+  if (fd->is_nugget_estim)
+    fd->nugget = S_2(0, 0) / (n - d) * (1 / _alpha - 1);
+
+  fd->var = S_2(0, 0) / (n - d) / _alpha;
 
   double log_S_2 = log(S_2(0, 0));
-
-  double log_marginal_lik = -sum(log(L.diag())) - sum(log(LX.diag())) - (m_X.n_rows - m_F.n_cols) / 2.0 * log_S_2;
+  double log_marginal_lik = -sum(log(L.diag())) - sum(log(LX.diag())) - (n - d) / 2.0 * log_S_2;
+  t0 = Bench::toc(bench, "lml = -Sum(log(diag(T))) - Sum(log(diag(TF)))...", t0);
   // arma::cout << " log_marginal_lik:" << log_marginal_lik << arma::endl;
 
   // Default prior params
   double a = 0.2;
-  double b = 1.0 / pow(m_X.n_rows, 1.0 / m_X.n_cols) * (a + 1.0);
+  double b = 1.0 / pow(n, 1.0 / d) * (a + d);
 
-  arma::vec CL = trans(max(m_X, 0) - min(m_X, 0)) / pow(m_X.n_rows, 1.0 / m_X.n_cols);
-  double t = arma::accu(CL % pow(_theta, -1.0)) + fd->nugget;
+  arma::vec CL = trans(max(m_X, 0) - min(m_X, 0)) / pow(n, 1.0 / d);
+  t0 = Bench::toc(bench, "CL = (max(X) - min(X)) / n^1/d", t0);
+  // arma::cout << " CL:" << CL << arma::endl;
+
+  double t = arma::accu(CL % pow(_theta, -1.0)) + fd->nugget / fd->sigma2;
+  // arma::cout << " t:" << t << arma::endl;
   double log_approx_ref_prior = -b * t + a * log(t);
   // arma::cout << " log_approx_ref_prior:" << log_approx_ref_prior << arma::endl;
 
@@ -403,8 +471,10 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
     //   Wb_ti=(L.transpose().triangularView<Upper>().solve(L.triangularView<Lower>().solve(dev_R_i))).transpose()-dev_R_i*R_inv_X_Xt_R_inv_X_inv_Xt_R_inv;
     //   ans[ti]=-0.5*Wb_ti.diagonal().sum()+(num_obs-q)/2.0*(output.transpose()*Wb_ti.transpose()*Q_output/S_2(0,0))(0,0);
     // }
+    t0 = Bench::tic();
     arma::vec ans = arma::ones(m_X.n_cols);
     arma::mat Q_output = trans(yt_R_inv) - R_inv_X_Xt_R_inv_X_inv_Xt_R_inv * m_y;
+    t0 = Bench::toc(bench, "Qo = YtRi - RiFFtRiFiFtRi * y", t0);
 
     arma::cube gradR = arma::cube(d, n, n);
     for (arma::uword i = 0; i < n; i++) {
@@ -412,9 +482,11 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
         gradR.slice(i).col(j) = R.at(i, j) * DlnCovDtheta(m_dX.col(i * n + j), _theta);
       }
     }
+    t0 = Bench::toc(bench, "gradR = R * dlnCov(dX)", t0);
 
     arma::mat Wb_k;
     for (arma::uword k = 0; k < m_X.n_cols; k++) {
+      t0 = Bench::tic();
       arma::mat gradR_k = arma::mat(n, n);
       for (arma::uword i = 0; i < n; i++) {
         gradR_k.at(i, i) = 0;
@@ -422,16 +494,18 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
           gradR_k.at(i, j) = gradR_k.at(j, i) = gradR.slice(i).col(j)[k];
         }
       }
+      t0 = Bench::toc(bench, "gradR_k = gradR[k]", t0);
 
       Wb_k = trans(solve(
                  trans(L), solve(L, gradR_k, LinearAlgebra::default_solve_opts), LinearAlgebra::default_solve_opts))
              - gradR_k * R_inv_X_Xt_R_inv_X_inv_Xt_R_inv;
-      ans[k] = -0.5 * sum(Wb_k.diag())
-               + (m_X.n_rows - m_F.n_cols) / 2.0 * (trans(m_y) * trans(Wb_k) * Q_output / S_2(0, 0))[0];
+      t0 = Bench::toc(bench, "Wb_k = gradR_k \\ T \\ Tt - gradR_k * RiFFtRiFiFtRi", t0);
+
+      ans[k] = -0.5 * sum(Wb_k.diag()) + (n - d) / 2.0 * (trans(m_y) * trans(Wb_k) * Q_output / S_2(0, 0))[0];
+      t0 = Bench::toc(bench, "ans[k] = Sum(diag(Wb_k)) + yt * Wb_kt * Qo / S2...", t0);
     }
     // arma::cout << " log_marginal_lik_deriv:" << -ans * pow(_theta,2) << arma::endl;
     // arma::cout << " log_approx_ref_prior_deriv:" <<  a*CL/t - b*CL << arma::endl;
-
     (*grad_out).head(d) = ans - (a * CL / t - b * CL) / pow(_theta, 2.0);
 
     arma::mat gradR_d = R / _alpha;
@@ -439,10 +513,9 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
     Wb_k = trans(
                solve(trans(L), solve(L, gradR_d, LinearAlgebra::default_solve_opts), LinearAlgebra::default_solve_opts))
            - gradR_d * R_inv_X_Xt_R_inv_X_inv_Xt_R_inv;
-    double ans_d = -0.5 * sum(Wb_k.diag())
-                   + (m_X.n_rows - m_F.n_cols) / 2.0 * (trans(m_y) * trans(Wb_k) * Q_output / S_2(0, 0))[0];
+    double ans_d = -0.5 * sum(Wb_k.diag()) + (n - d) / 2.0 * (trans(m_y) * trans(Wb_k) * Q_output / S_2(0, 0))[0];
 
-    (*grad_out).at(d) = ans_d;  //- (a * 1.0 / t - b * 1.0);
+    (*grad_out).at(d) = ans_d - (a / t - b) / pow(_alpha, 2.0);
 
     // arma::cout << " grad_out:" << *grad_out << arma::endl;
   }
@@ -451,8 +524,9 @@ double NuggetKriging::_logMargPost(const arma::vec& _theta_alpha,
   return (log_marginal_lik + log_approx_ref_prior);
 }
 
-LIBKRIGING_EXPORT std::tuple<double, arma::vec> NuggetKriging::logMargPostFun(const arma::vec& _theta,
-                                                                              const bool _grad) {
+LIBKRIGING_EXPORT std::tuple<double, arma::vec> NuggetKriging::logMargPostFun(const arma::vec& _theta_alpha,
+                                                                              const bool _grad,
+                                                                              const bool _bench) {
   arma::mat T;
   arma::mat M;
   arma::colvec z;
@@ -460,15 +534,32 @@ LIBKRIGING_EXPORT std::tuple<double, arma::vec> NuggetKriging::logMargPostFun(co
   double sigma2{};
   double nugget{};
   double var{};
-  NuggetKriging::OKModel okm_data{T, M, z, beta, true, sigma2, true, nugget, false, var};
+  NuggetKriging::OKModel okm_data{T, M, z, beta, true, sigma2, true, nugget, true, var};
 
   double lmp = -1;
   arma::vec grad;
-  if (_grad) {
-    grad = arma::vec(_theta.n_elem);
-    lmp = _logMargPost(_theta, &grad, &okm_data);
-  } else
-    lmp = _logMargPost(_theta, nullptr, &okm_data);
+
+  if (_bench) {
+    std::map<std::string, double> bench;
+    if (_grad) {
+      grad = arma::vec(_theta_alpha.n_elem);
+      lmp = _logMargPost(_theta_alpha, &grad, &okm_data, &bench);
+    } else
+      lmp = _logMargPost(_theta_alpha, nullptr, &okm_data, &bench);
+
+    size_t num = 0;
+    for (auto& kv : bench)
+      num = std::max(kv.first.size(), num);
+    for (auto& kv : bench)
+      arma::cout << "| " << Bench::pad(kv.first, num, ' ') << " | " << kv.second << " |" << arma::endl;
+
+  } else {
+    if (_grad) {
+      grad = arma::vec(_theta_alpha.n_elem);
+      lmp = _logMargPost(_theta_alpha, &grad, &okm_data, nullptr);
+    } else
+      lmp = _logMargPost(_theta_alpha, nullptr, &okm_data, nullptr);
+  }
 
   return std::make_tuple(lmp, std::move(grad));
 }
@@ -478,7 +569,7 @@ LIBKRIGING_EXPORT double NuggetKriging::logLikelihood() {
   arma::vec _theta_alpha = arma::vec(d + 1);
   _theta_alpha.head(d) = m_theta;
   _theta_alpha.at(d) = m_sigma2 / (m_sigma2 + m_nugget);
-  return std::get<0>(NuggetKriging::logLikelihoodFun(_theta_alpha, false));
+  return std::get<0>(NuggetKriging::logLikelihoodFun(_theta_alpha, false, false));
 }
 
 LIBKRIGING_EXPORT double NuggetKriging::logMargPost() {
@@ -486,9 +577,34 @@ LIBKRIGING_EXPORT double NuggetKriging::logMargPost() {
   arma::vec _theta_alpha = arma::vec(d + 1);
   _theta_alpha.head(d) = m_theta;
   _theta_alpha.at(d) = m_sigma2 / (m_sigma2 + m_nugget);
-  return std::get<0>(NuggetKriging::logMargPostFun(_theta_alpha, false));
+  return std::get<0>(NuggetKriging::logMargPostFun(_theta_alpha, false, false));
 }
 
+std::function<arma::vec(const arma::vec&)> NuggetKriging::reparam_to = [](const arma::vec& _theta_alpha) {
+  arma::vec _theta_malpha = _theta_alpha;
+  const arma::uword d = _theta_alpha.n_elem - 1;
+  _theta_malpha.at(d) = 1 + alpha_lower - _theta_malpha.at(d);
+  return Optim::reparam_to(_theta_malpha);
+};
+
+std::function<arma::vec(const arma::vec&)> NuggetKriging::reparam_from = [](const arma::vec& _gamma) {
+  arma::vec _theta_alpha = Optim::reparam_from(_gamma);
+  const arma::uword d = _theta_alpha.n_elem - 1;
+  _theta_alpha.at(d) = 1 + alpha_lower - _theta_alpha.at(d);
+  return _theta_alpha;
+};
+
+std::function<arma::vec(const arma::vec&, const arma::vec&)> NuggetKriging::reparam_from_deriv
+    = [](const arma::vec& _theta_alpha, const arma::vec& _grad) {
+        arma::vec D_theta_alpha = arma::conv_to<arma::colvec>::from(-_grad % _theta_alpha);
+        const arma::uword d = D_theta_alpha.n_elem - 1;
+        D_theta_alpha.at(d) = (1 + alpha_lower - _theta_alpha.at(d)) * _grad.at(d);
+        return D_theta_alpha;
+      };
+
+double NuggetKriging::alpha_upper = 1.0;
+double NuggetKriging::alpha_lower = 1E-3;
+
 /** Fit the kriging object on (X,y):
  * @param y is n length column vector of output
  * @param X is n*d matrix of input
@@ -516,13 +632,24 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       fit_ofn = CacheFunction([this](const arma::vec& _gamma, arma::vec* grad_out, NuggetKriging::OKModel* okm_data) {
         // Change variable for opt: . -> 1/exp(.)
         // DEBUG: if (Optim::log_level>3) arma::cout << "> gamma: " << _gamma << arma::endl;
-        const arma::vec _theta_alpha = Optim::reparam_from(_gamma);
+        const arma::vec _theta_alpha = NuggetKriging::reparam_from(_gamma);
         // DEBUG: if (Optim::log_level>3) arma::cout << "> theta_alpha: " << _theta_alpha << arma::endl;
-        double ll = this->_logLikelihood(_theta_alpha, grad_out, okm_data);
-        // DEBUG: if (Optim::log_level>3) arma::cout << "  > ll: " << ll << arma::endl;
+        double ll = this->_logLikelihood(_theta_alpha, grad_out, okm_data, nullptr);
+        // DEBUG: if (Optim::log_level>3) arma::cout << "  > -ll: " << -ll << arma::endl;
         if (grad_out != nullptr) {
-          // DEBUG: if (Optim::log_level>3) arma::cout << "  > grad ll: " << grad_out << arma::endl;
-          *grad_out = -Optim::reparam_from_deriv(_theta_alpha, *grad_out);
+          *grad_out = -NuggetKriging::reparam_from_deriv(_theta_alpha, *grad_out);
+          // DEBUG:
+          // if (Optim::log_level>3) {
+          //  arma::cout << "  > grad -ll: " << *grad_out << arma::endl;
+          //  //// Check with numerical gradient:
+          //  //for (size_t i = 0; i <_gamma.n_elem; i++) {
+          //  //  arma::vec eps = arma::zeros(_gamma.n_elem);
+          //  //  eps(i) = 0.000001;
+          //  //  const arma::vec _theta_alpha_eps = reparam_from(_gamma + eps);
+          //  //  double ll_eps = this->_logLikelihood(_theta_alpha_eps, nullptr, okm_data);
+          //  //  arma::cout << "  > num_grad -ll: " << -(ll_eps - ll)/0.000001 << arma::endl;
+          //  //}
+          //}
         }
         return -ll;
       });
@@ -530,7 +657,7 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       fit_ofn = CacheFunction([this](const arma::vec& _gamma, arma::vec* grad_out, NuggetKriging::OKModel* okm_data) {
         const arma::vec _theta_alpha = _gamma;
         // DEBUG: if (Optim::log_level>3) arma::cout << "> theta_alpha: " << _theta_alpha << arma::endl;
-        double ll = this->_logLikelihood(_theta_alpha, grad_out, okm_data);
+        double ll = this->_logLikelihood(_theta_alpha, grad_out, okm_data, nullptr);
         // DEBUG: if (Optim::log_level>3) arma::cout << "  > ll: " << ll << arma::endl;
         if (grad_out != nullptr) {
           // DEBUG: if (Optim::log_level>3) arma::cout << "  > grad ll: " << grad_out << arma::endl;
@@ -546,13 +673,13 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       fit_ofn = CacheFunction([this](const arma::vec& _gamma, arma::vec* grad_out, NuggetKriging::OKModel* okm_data) {
         // Change variable for opt: . -> 1/exp(.)
         // DEBUG: if (Optim::log_level>3) arma::cout << "> gamma: " << _gamma << arma::endl;
-        const arma::vec _theta_alpha = Optim::reparam_from(_gamma);
+        const arma::vec _theta_alpha = NuggetKriging::reparam_from(_gamma);
         // DEBUG: if (Optim::log_level>3) arma::cout << "> theta_alpha: " << _theta_alpha << arma::endl;
-        double lmp = this->_logMargPost(_theta_alpha, grad_out, okm_data);
+        double lmp = this->_logMargPost(_theta_alpha, grad_out, okm_data, nullptr);
         // DEBUG: if (Optim::log_level>3) arma::cout << "  > lmp: " << lmp << arma::endl;
         if (grad_out != nullptr) {
           // DEBUG: if (Optim::log_level>3) arma::cout << "  > grad lmp: " << grad_out << arma::endl;
-          *grad_out = -Optim::reparam_from_deriv(_theta_alpha, *grad_out);
+          *grad_out = -NuggetKriging::reparam_from_deriv(_theta_alpha, *grad_out);
         }
         return -lmp;
       });
@@ -560,7 +687,7 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       fit_ofn = CacheFunction([this](const arma::vec& _gamma, arma::vec* grad_out, NuggetKriging::OKModel* okm_data) {
         const arma::vec _theta_alpha = _gamma;
         // DEBUG: if (Optim::log_level>3) arma::cout << "> theta_alpha: " << _theta_alpha << arma::endl;
-        double lmp = this->_logMargPost(_theta_alpha, grad_out, okm_data);
+        double lmp = this->_logMargPost(_theta_alpha, grad_out, okm_data, nullptr);
         // DEBUG: if (Optim::log_level>3) arma::cout << "  > lmp: " << lmp << arma::endl;
         if (grad_out != nullptr) {
           // DEBUG: if (Optim::log_level>3) arma::cout << "  > grad lmp: " << grad_out << arma::endl;
@@ -643,41 +770,42 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
     arma::mat M;
     arma::colvec z;
     arma::colvec beta;
+    bool is_beta_estim = parameters.is_beta_estim;
     if (parameters.beta.has_value()) {
       beta = parameters.beta.value();
       if (m_normalize)
         beta /= scaleY;
+    } else {
+      is_beta_estim = true;  // force estim if no value given
     }
     double sigma2 = -1;
+    bool is_sigma2_estim = parameters.is_sigma2_estim;
     if (parameters.sigma2.has_value()) {
       sigma2 = parameters.sigma2.value()[0];  // otherwise sigma2 will be re-calculated using given theta
       if (m_normalize)
         sigma2 /= (scaleY * scaleY);
+    } else {
+      is_sigma2_estim = true;  // force estim if no value given
     }
     double nugget = -1;
+    bool is_nugget_estim = parameters.is_nugget_estim;
     if (parameters.nugget.has_value()) {
       nugget = parameters.nugget.value()[0];
       if (m_normalize)
         nugget /= (scaleY * scaleY);
+    } else {
+      is_nugget_estim = true;  // force estim if no value given
     }
 
-    NuggetKriging::OKModel okm_data{T,
-                                    M,
-                                    z,
-                                    beta,
-                                    parameters.is_beta_estim,
-                                    sigma2,
-                                    parameters.is_sigma2_estim,
-                                    nugget,
-                                    parameters.is_nugget_estim,
-                                    nugget + sigma2};
+    NuggetKriging::OKModel okm_data{
+        T, M, z, beta, is_beta_estim, sigma2, is_sigma2_estim, nugget, is_nugget_estim, nugget + sigma2};
 
     arma::vec gamma_tmp = arma::vec(d + 1);
     gamma_tmp.head(d) = m_theta;
     gamma_tmp.at(d) = sigma2 / (nugget + sigma2);
     if (Optim::reparametrize) {
       gamma_tmp.head(d) = Optim::reparam_to(m_theta);
-      gamma_tmp.at(d) = Optim::reparam_to_(sigma2 / (nugget + sigma2));
+      gamma_tmp.at(d) = Optim::reparam_to_(1 + alpha_lower - sigma2 / (nugget + sigma2));
     }
 
     /* double min_ofn_tmp = */ fit_ofn(gamma_tmp, nullptr, &okm_data);
@@ -685,12 +813,24 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
     m_T = std::move(okm_data.T);
     m_M = std::move(okm_data.M);
     m_z = std::move(okm_data.z);
-    m_beta = std::move(okm_data.beta);
-    m_est_beta = parameters.is_beta_estim;
-    m_sigma2 = okm_data.sigma2;
-    m_est_sigma2 = parameters.is_sigma2_estim;
-    m_nugget = okm_data.nugget;
-    m_est_nugget = parameters.is_nugget_estim;
+    m_est_beta = is_beta_estim;
+    if (m_est_beta) {
+      m_beta = std::move(okm_data.beta);
+    } else {
+      m_beta = beta;
+    }
+    m_est_sigma2 = is_sigma2_estim;
+    if (m_est_sigma2) {
+      m_sigma2 = okm_data.sigma2;
+    } else {
+      m_sigma2 = sigma2;
+    }
+    m_est_nugget = is_nugget_estim;
+    if (m_est_nugget) {
+      m_nugget = okm_data.nugget;
+    } else {
+      m_nugget = nugget;
+    }
 
   } else if (optim.rfind("BFGS", 0) == 0) {
     Random::init();
@@ -744,9 +884,6 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
     }
     // arma::cout << "theta0:" << theta0 << arma::endl;
 
-    double alpha_upper = 1.0;
-    double alpha_lower = 1E-3;
-
     arma::vec alpha0;
     if (parameters.sigma2.has_value() && parameters.nugget.has_value()) {
       alpha0 = arma::vec(parameters.sigma2.value().n_elem * parameters.nugget.value().n_elem);
@@ -755,14 +892,14 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
           if ((parameters.sigma2.value()[i] < 0) || (parameters.nugget.value()[j] < 0)
               || (parameters.sigma2.value()[i] + parameters.nugget.value()[j] < 0))
             alpha0[i + j * parameters.sigma2.value().n_elem]
-                = alpha_lower + (alpha_upper - alpha_lower) * Random::randu();
+                = alpha_lower + (alpha_upper - alpha_lower) * (1 - std::pow(Random::randu(), 3.0));
           else
             alpha0[i + j * parameters.sigma2.value().n_elem]
                 = parameters.sigma2.value()[i] / (parameters.sigma2.value()[i] + parameters.nugget.value()[j]);
         }
       }
     } else {
-      alpha0 = alpha_lower + (alpha_upper - alpha_lower) * Random::randu_vec(theta0.n_rows);
+      alpha0 = alpha_lower + (alpha_upper - alpha_lower) * (1 - arma::pow(Random::randu_vec(theta0.n_rows), 3.0));
     }
     // arma::cout << "alpha0:" << alpha0 << arma::endl;
 
@@ -774,8 +911,11 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
     gamma_upper.at(d) = alpha_upper;
     if (Optim::reparametrize) {
       arma::vec gamma_lower_tmp = gamma_lower;
-      gamma_lower = Optim::reparam_to(gamma_upper);
-      gamma_upper = Optim::reparam_to(gamma_lower_tmp);
+      gamma_lower = NuggetKriging::reparam_to(gamma_upper);
+      gamma_upper = NuggetKriging::reparam_to(gamma_lower_tmp);
+      double gamma_lower_at_d = gamma_lower.at(d);
+      gamma_lower.at(d) = gamma_upper.at(d);
+      gamma_upper.at(d) = gamma_lower_at_d;
     }
 
     double min_ofn = std::numeric_limits<double>::infinity();
@@ -785,7 +925,7 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       gamma_tmp.head(d) = theta0.row(i).t();
       gamma_tmp.at(d) = alpha0[i % alpha0.n_elem];
       if (Optim::reparametrize) {
-        gamma_tmp = Optim::reparam_to(gamma_tmp);
+        gamma_tmp = NuggetKriging::reparam_to(gamma_tmp);
       }
 
       gamma_lower = arma::min(gamma_tmp, gamma_lower);
@@ -843,10 +983,12 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       optimizer.iprint = Optim::log_level - 2;
       optimizer.max_iter = Optim::max_iteration;
       optimizer.pgtol = Optim::gradient_tolerance;
-      optimizer.factr = Optim::objective_rel_tolerance / 1E-13;
+      optimizer.factr = Optim::objective_rel_tolerance / 1E-11;
       arma::ivec bounds_type{d + 1, arma::fill::value(2)};  // means both upper & lower bounds
+
       int retry = 0;
       while (retry <= Optim::max_restart) {
+        arma::vec gamma_0 = gamma_tmp;
         auto result = optimizer.minimize(
             [&okm_data, &fit_ofn](const arma::vec& vals_inp, arma::vec& grad_out) -> double {
               return fit_ofn(vals_inp, &grad_out, &okm_data);
@@ -855,13 +997,27 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
             gamma_lower.memptr(),
             gamma_upper.memptr(),
             bounds_type.memptr());
+
+        if (Optim::log_level > 0) {
+          arma::cout << "     iterations: " << result.num_iters << arma::endl;
+          if (Optim::reparametrize) {
+            arma::cout << "     start_point: " << NuggetKriging::reparam_from(gamma_0).t() << " ";
+            arma::cout << "     solution: " << NuggetKriging::reparam_from(gamma_tmp).t() << " ";
+          } else {
+            arma::cout << "     start_point: " << gamma_0.t() << " ";
+            arma::cout << "     solution: " << gamma_tmp.t() << " ";
+          }
+        }
+
         double sol_to_lb_theta = arma::min(arma::abs(gamma_tmp.head(d) - gamma_lower.head(d)));
         double sol_to_ub_theta = arma::min(arma::abs(gamma_tmp.head(d) - gamma_upper.head(d)));
         double sol_to_b_theta
             = Optim::reparametrize ? sol_to_ub_theta : sol_to_lb_theta;  // just consider theta lower bound
-        double sol_to_b_alpha = Optim::reparametrize
-                                    ? std::abs(gamma_tmp.at(d) - gamma_upper.at(d))
-                                    : std::abs(gamma_tmp.at(d) - gamma_lower.at(d));  // just consider alpha lower bound
+        double sol_to_b_alpha
+            = std::abs(gamma_tmp.at(d) - gamma_lower.at(d));  // just consider alpha lower bound
+                                                              // Optim::reparametrize
+                                                              //    ? std::abs(gamma_tmp.at(d) - gamma_upper.at(d))
+                                                              //    : std::abs(gamma_tmp.at(d) - gamma_lower.at(d));
         double sol_to_b = sol_to_b_theta < sol_to_b_alpha ? sol_to_b_theta : sol_to_b_alpha;
         if ((retry < Optim::max_restart)       //&& (result.num_iters <= 2 * d)
             && ((sol_to_b < arma::datum::eps)  // we fastly converged to one bound
@@ -869,16 +1025,14 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
           gamma_tmp.head(d)
               = (theta0.row(i).t() + theta_lower)
                 / pow(2.0, retry + 1);  // so, re-use previous starting point and change it to middle-point
-          gamma_tmp.at(d) = alpha0[i % alpha0.n_elem];
-          if (Optim::log_level > 0)
-            arma::cout << "    start_point: " << gamma_tmp.t() << " ";
+          gamma_tmp.at(d) = alpha_upper - (alpha0[i % alpha0.n_elem] + alpha_upper) / pow(2.0, retry + 1);
+
           if (Optim::reparametrize)
-            gamma_tmp = Optim::reparam_to(gamma_tmp);
-          if (Optim::log_level > 0) {
-            arma::cout << "    iterations: " << result.num_iters << arma::endl;
-          }
+            gamma_tmp = NuggetKriging::reparam_to(gamma_tmp);
+
           gamma_lower = arma::min(gamma_tmp, gamma_lower);
           gamma_upper = arma::max(gamma_tmp, gamma_upper);
+
           retry++;
         } else {
           if (Optim::log_level > 1)
@@ -893,7 +1047,7 @@ LIBKRIGING_EXPORT void NuggetKriging::fit(const arma::colvec& y,
       if (Optim::log_level > 0) {
         arma::cout << "  best objective: " << min_ofn_tmp << arma::endl;
         if (Optim::reparametrize)
-          arma::cout << "  best solution: " << Optim::reparam_from(gamma_tmp) << " ";
+          arma::cout << "  best solution: " << NuggetKriging::reparam_from(gamma_tmp) << " ";
         else
           arma::cout << "  best solution: " << gamma_tmp << "";
       }
@@ -1242,3 +1396,86 @@ LIBKRIGING_EXPORT std::string NuggetKriging::summary() const {
   }
   return oss.str();
 }
+
+void NuggetKriging::save(const std::string filename) const {
+  const auto appflag = arma::hdf5_opts::append;
+
+  uint32_t version = 1;
+  saveToHdf5(version, arma::hdf5_name(filename, "version", arma::hdf5_opts::none));
+  saveToHdf5(std::string("NuggetKriging"), arma::hdf5_name(filename, "content", appflag));
+
+  // Cov_pow & std::function embedded by make_Cov
+  saveToHdf5(m_covType, arma::hdf5_name(filename, "covType", appflag));
+  m_X.save(arma::hdf5_name(filename, "X", appflag));
+  m_centerX.save(arma::hdf5_name(filename, "centerX", appflag));
+  m_scaleX.save(arma::hdf5_name(filename, "scaleX", appflag));
+  m_y.save(arma::hdf5_name(filename, "y", appflag));
+  saveToHdf5(m_centerY, arma::hdf5_name(filename, "centerY", appflag));
+  saveToHdf5(m_scaleY, arma::hdf5_name(filename, "scaleY", appflag));
+  saveToHdf5(m_normalize, arma::hdf5_name(filename, "normalize", appflag));
+
+  saveToHdf5(Trend::toString(m_regmodel), arma::hdf5_name(filename, "regmodel", appflag));
+  saveToHdf5(m_optim, arma::hdf5_name(filename, "optim", appflag));
+  saveToHdf5(m_objective, arma::hdf5_name(filename, "objective", appflag));
+  m_dX.save(arma::hdf5_name(filename, "dX", appflag));
+  m_F.save(arma::hdf5_name(filename, "F", appflag));
+  m_T.save(arma::hdf5_name(filename, "T", appflag));
+  m_M.save(arma::hdf5_name(filename, "M", appflag));
+  m_z.save(arma::hdf5_name(filename, "z", appflag));
+  m_beta.save(arma::hdf5_name(filename, "beta", appflag));
+  saveToHdf5(m_est_beta, arma::hdf5_name(filename, "est_beta", appflag));
+  m_theta.save(arma::hdf5_name(filename, "theta", appflag));
+  saveToHdf5(m_est_theta, arma::hdf5_name(filename, "est_theta", appflag));
+  saveToHdf5(m_sigma2, arma::hdf5_name(filename, "sigma2", appflag));
+  saveToHdf5(m_est_sigma2, arma::hdf5_name(filename, "est_sigma2", appflag));
+  saveToHdf5(m_nugget, arma::hdf5_name(filename, "nugget", appflag));
+  saveToHdf5(m_est_nugget, arma::hdf5_name(filename, "est_nugget", appflag));
+}
+
+NuggetKriging NuggetKriging::load(const std::string filename) {
+  uint32_t version;
+  loadFromHdf5(version, arma::hdf5_name(filename, "version"));
+  if (version != 1) {
+    throw std::runtime_error(asString("Bad version to load from '", filename, "'; found ", version, ", requires 1"));
+  }
+  std::string content;
+  loadFromHdf5(content, arma::hdf5_name(filename, "content"));
+  if (content != "NuggetKriging") {
+    throw std::runtime_error(
+        asString("Bad content to load from '", filename, "'; found '", content, "', requires 'NuggetKriging'"));
+  }
+
+  std::string covType;
+  loadFromHdf5(covType, arma::hdf5_name(filename, "covType"));
+  NuggetKriging kr(covType);  // Cov_pow & std::function embedded by make_Cov
+
+  kr.m_X.load(arma::hdf5_name(filename, "X"));
+  kr.m_centerX.load(arma::hdf5_name(filename, "centerX"));
+  kr.m_scaleX.load(arma::hdf5_name(filename, "scaleX"));
+  kr.m_y.load(arma::hdf5_name(filename, "y"));
+  loadFromHdf5(kr.m_centerY, arma::hdf5_name(filename, "centerY"));
+  loadFromHdf5(kr.m_scaleY, arma::hdf5_name(filename, "scaleY"));
+  loadFromHdf5(kr.m_normalize, arma::hdf5_name(filename, "normalize"));
+
+  std::string model;
+  loadFromHdf5(model, arma::hdf5_name(filename, "regmodel"));
+  kr.m_regmodel = Trend::fromString(model);
+
+  loadFromHdf5(kr.m_optim, arma::hdf5_name(filename, "optim"));
+  loadFromHdf5(kr.m_objective, arma::hdf5_name(filename, "objective"));
+  kr.m_dX.load(arma::hdf5_name(filename, "dX"));
+  kr.m_F.load(arma::hdf5_name(filename, "F"));
+  kr.m_T.load(arma::hdf5_name(filename, "T"));
+  kr.m_M.load(arma::hdf5_name(filename, "M"));
+  kr.m_z.load(arma::hdf5_name(filename, "z"));
+  kr.m_beta.load(arma::hdf5_name(filename, "beta"));
+  loadFromHdf5(kr.m_est_beta, arma::hdf5_name(filename, "est_beta"));
+  kr.m_theta.load(arma::hdf5_name(filename, "theta"));
+  loadFromHdf5(kr.m_est_theta, arma::hdf5_name(filename, "est_theta"));
+  loadFromHdf5(kr.m_sigma2, arma::hdf5_name(filename, "sigma2"));
+  loadFromHdf5(kr.m_est_sigma2, arma::hdf5_name(filename, "est_sigma2"));
+  loadFromHdf5(kr.m_nugget, arma::hdf5_name(filename, "nugget"));
+  loadFromHdf5(kr.m_est_nugget, arma::hdf5_name(filename, "est_nugget"));
+
+  return kr;
+}
