@@ -33,7 +33,7 @@ lmp_rgasp = function(X, model=k) {if (!is.matrix(X)) X = matrix(X,ncol=1);
                                         R0=model@R0,X=model@X, zero_mean=model@zero_mean,output=model@output, 
                                         CL=model@CL, 
                                         a=0.2,
-                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@output))[2]),
+                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@input))[2]),
                                         kernel_type=rep(as.integer(3),ncol(X)),alpha=model@alpha
                                         )
                       y})}
@@ -74,7 +74,7 @@ dlmp_rgasp = function(X, model=k) {if (!is.matrix(X)) X = matrix(X,ncol=1);
 #    print(RobustGaSP:::log_approx_ref_prior_deriv(param=(x),nugget=0, nugget_est=model@nugget.est, 
 #                                        CL=model@CL, 
 #                                        a=0.2,
-#                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@output))[2])))
+#                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@input))[2])))
 
 
                       #y=-logMargPostFun(r,matrix(unlist(x),ncol=2))$logMargPost
@@ -82,7 +82,7 @@ dlmp_rgasp = function(X, model=k) {if (!is.matrix(X)) X = matrix(X,ncol=1);
                                         R0=model@R0,X=model@X, zero_mean=model@zero_mean,output=model@output, 
                                         CL=model@CL, 
                                         a=0.2,
-                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@output))[2]),
+                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@input))[2]),
                                         kernel_type=rep(as.integer(3),ncol(X)),alpha=model@alpha
                                         )
                       y})}
@@ -215,7 +215,7 @@ lmp_rgasp = function(X, model=k) {if (!is.matrix(X)) X = matrix(X,ncol=2);
                                         R0=model@R0,X=model@X, zero_mean=model@zero_mean,output=model@output, 
                                         CL=model@CL, 
                                         a=0.2,
-                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@output))[2]),
+                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@input))[2]),
                                         kernel_type=rep(as.integer(3),ncol(X)),alpha=model@alpha
                                         )
                       y})}
@@ -251,7 +251,7 @@ dlmp_rgasp = function(X, model=k) {if (!is.matrix(X)) X = matrix(X,ncol=2);
                                         R0=model@R0,X=model@X, zero_mean=model@zero_mean,output=model@output, 
                                         CL=model@CL, 
                                         a=0.2,
-                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@output))[2]),
+                                        b=1/(length(model@output))^{1/dim(as.matrix(model@input))[2]}*(0.2+dim(as.matrix(model@input))[2]),
                                         kernel_type=rep(as.integer(3),ncol(X)),alpha=model@alpha
                                         )
                       y})}
