@@ -2,6 +2,9 @@
 
 packages = commandArgs(trailingOnly=TRUE)
 
+if (.Platform$OS.type=="windows")
+    options(pkgType = "win.binary") # Prefer windows binary if available (even if not latest version)
+
 for (lib in packages) {
 
     install.packages(lib, repos='https://cloud.r-project.org');
@@ -10,4 +13,4 @@ for (lib in packages) {
         cat(paste("\n#########################\nCannot install", lib, "\n#########################\n\n"))
         quit(status=1, save='no')
     }
-}
\ No newline at end of file
+}
